package com.qianhe.system.utils;

import com.idrsolutions.image.png.PngCompressor;
import net.coobird.thumbnailator.Thumbnails;
import net.coobird.thumbnailator.name.Rename;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.io.*;
import java.util.ArrayList;
import java.util.List;

/**
 * 图片压缩工具
 */
@Component
public class ImageThumbnailsUtil {

    @Value("${ruoyi.profile}")
    private String profile;

    /***
     * 压缩jpg、jpeg格式图片
     * @param inputImagePath
     * @param outputImagePath
     * @throws IOException
     */
    public void compressImage(String inputImagePath, String outputImagePath) throws IOException{
        // 指定原始图片路径
        inputImagePath = profile + inputImagePath.replace("/profile","");
        // 设置目标图片保存路径及名称
        outputImagePath = profile + outputImagePath.replace("/profile","");

        File file = new File(inputImagePath); // 源文件
        File outfile = new File(outputImagePath); // 目标输出文件，可与源文件一致，一致会覆盖
        PngCompressor.compress(file, outfile);

//        // 创建输入流对象
//        FileInputStream fis = new FileInputStream(new File(inputImagePath));
//
//        // 获取图片信息
//        BufferedImage image = ImageIO.read(fis);
//
//        // 创建输出流对象
//        FileOutputStream fos = new FileOutputStream(new File(outputImagePath));
//
//        // 将图片写入到输出流中并进行压缩处理
//        JPEGImageEncoder encoder = JPEGCodec.createJPEGEncoder(fos);
//        encoder.encode(image);
//
//        // 关闭输入、输出流
//        fis.close();
//        fos.close();
    }

    /***
     * 压缩PNG格式图片
     * @param inputImagePath
     * @param outputImagePath
     * @throws IOException
     */
    public void compressImagePNG(String inputImagePath, String outputImagePath) throws IOException{
        // 指定原始图片路径
        inputImagePath = profile + inputImagePath.replace("/profile","");
        outputImagePath = profile + outputImagePath.replace("/profile","");
        File file = new File(inputImagePath); // 源文件
        File outfile = new File(outputImagePath); // 目标输出文件，可与源文件一致，一致会覆盖
        PngCompressor.compress(file, outfile);
    }



    // 图片默认缩放比率
    private static final double DEFAULT_SCALE = 0.8d;

    // 缩略图后缀
    private static final String SUFFIX = "-thumbnail";


    /**
     * 生成缩略图到指定的目录
     *
     * @param path  目录
     * @param files 要生成缩略图的文件列表
     * @throws IOException
     */
    public static List<String> generateThumbnail2Directory(String path, String... files) throws IOException {
        return generateThumbnail2Directory(DEFAULT_SCALE, path, files);
    }

    /**
     * 生成缩略图到指定的目录
     *
     * @param scale    图片缩放率
     * @param pathname 缩略图保存目录
     * @param files    要生成缩略图的文件列表
     * @throws IOException
     */
    public static List<String> generateThumbnail2Directory(double scale, String pathname, String... files) throws IOException {
        Thumbnails.of(files)
                // 图片缩放率，不能和size()一起使用
                .scale(scale)
                // 缩略图保存目录,该目录需存在，否则报错
                .toFiles(new File(pathname), Rename.NO_CHANGE);
        List<String> list = new ArrayList<>(files.length);
        for (String file : files) {
            list.add(appendSuffix(file, SUFFIX));
        }
        return list;
    }

    public static void main(String[] args) throws IOException {
        generateDirectoryThumbnail("D:/qianhe/file/送水/水韵照片/2");
    }
    /**
     * 将指定目录下所有图片生成缩略图
     *
     * @param pathname 文件目录
     */
    public static void generateDirectoryThumbnail(String pathname) throws IOException {
        generateDirectoryThumbnail(pathname, DEFAULT_SCALE);
    }

    /**
     * 将指定目录下所有图片生成缩略图
     *
     * @param pathname 文件目录
     */
    public static void generateDirectoryThumbnail(String pathname, double scale) throws IOException {
        File[] files = new File(pathname).listFiles();
        compressRecurse(files, pathname);
    }

    /**
     * 文件追加后缀
     *
     * @param fileName 原文件名
     * @param suffix   文件后缀
     * @return
     */
    public static String appendSuffix(String fileName, String suffix) {
        String newFileName = "";

        int indexOfDot = fileName.lastIndexOf('.');

        if (indexOfDot != -1) {
            newFileName = fileName.substring(0, indexOfDot);
            newFileName += suffix;
            newFileName += fileName.substring(indexOfDot);
        } else {
            newFileName = fileName + suffix;
        }

        return newFileName;
    }


    private static void compressRecurse(File[] files, String pathname) throws IOException {
        for (File file : files) {
            // 目录
            if (file.isDirectory()) {
                File[] subFiles = file.listFiles();
                compressRecurse(subFiles, pathname + File.separator + file.getName());
            } else {
                // 文件包含压缩文件后缀或非图片格式，则不再压缩
                String extension = getFileExtention(file.getName());
                if (!file.getName().contains(SUFFIX) && isImage(extension)) {
                    generateThumbnail2Directory(pathname, file.getAbsolutePath());
                }
            }
        }
    }

    /**
     * 根据文件扩展名判断文件是否图片格式
     *
     * @param extension 文件扩展名
     * @return
     */
    public static boolean isImage(String extension) {
        String[] imageExtension = new String[]{"jpeg", "jpg", "gif", "bmp", "png"};

        for (String e : imageExtension) if (extension.toLowerCase().equals(e)) return true;

        return false;
    }

    public static String getFileExtention(String fileName) {
        String extension = fileName.substring(fileName.lastIndexOf(".") + 1);
        return extension;
    }


}
